/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * PostgreSQL CDR logger
 * by Chris Maj <cmaj@freedomcorpse.info>
 *
 * Borrows heavily from MySQL CDR logger
 * by James Sharp <jsharp@psychoses.org>
 *
 * With Mad Props to Mark Spencer
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License.
 *
 */

#include <sys/types.h>
#include <asterisk/config.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/cdr.h>
#include <asterisk/module.h>
#include <asterisk/logger.h>
#include "../asterisk.h"

#include <stdio.h>
#include <string.h>

#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include "postgresql/libpq-fe.h"

#define DATE_FORMAT "%Y-%m-%d %T"

static char *desc = "PostgreSQL CDR Backend";
static char *name = "postgres";
static char *config = "cdr_postgres.conf";

static PGconn *pg;

static int pg_log(struct ast_cdr *cdr)
{
    struct tm tm;
    struct timeval tv;
    struct timezone tz;
    char *sqlcmd, timestr[128];
    time_t t;
    PGresult *res;

    sqlcmd = (char *)malloc(2048);
    memset(sqlcmd,0,2048);

    gettimeofday(&tv,&tz);
    t = tv.tv_sec;
    localtime_r(&t,&tm);
    strftime(timestr,128,DATE_FORMAT,&tm);

    ast_log(LOG_DEBUG,"cdr_postgres: inserting a CDR record.\n");
    sprintf(sqlcmd,"INSERT INTO cdr (calldate,clid,src,dst,dcontext,channel,dstchannel,lastapp,lastdata,duration,billsec,disposition,amaflags,accountcode) VALUES ('%s','%s','%s','%s','%s', '%s','%s','%s','%s',%i,%i,%i,%i,'%s')",timestr,cdr->clid,cdr->src, cdr->dst, cdr->dcontext,cdr->channel, cdr->dstchannel, cdr->lastapp, cdr->lastdata,cdr->duration,cdr->billsec,cdr->disposition,cdr->amaflags, cdr->accountcode);

    ast_log(LOG_DEBUG,"cdr_postgres: SQL command as follows:  %s\n",sqlcmd);

    res = PQexec(pg,sqlcmd);
    if (!res)
    {
        ast_log(LOG_ERROR,"Failed to insert into database.");
        free(sqlcmd);
        return -1;
    }

    free(sqlcmd);

    return 0;
}



char *description(void)
{
    return desc;
}

int unload_module(void)
{
    PQfinish(pg);
    ast_cdr_unregister(name);
    return 0;
}

int load_module(void)
{
    int res;
    struct ast_config *cfg;
    struct ast_variable *var;

    char *conn;

    cfg = ast_load(config);
    if (!cfg) {
        ast_log(LOG_WARNING, "Unable to load config for PostgreSQL CDRs: %s\n", config);
        return 0;
    }

    var = ast_variable_browse(cfg, "global");
    if (!var) {
        /* nothing configured */
        return 0;
    }

    conn = ast_variable_retrieve(cfg,"global","conn");
    ast_log(LOG_DEBUG,"cdr_postgres: got conn of %s\n",conn);

    if (conn == NULL)
    {
        ast_log(LOG_ERROR,"Connection string not specified.\n");
        return -1;
    }

    pg = PQconnectdb(conn);

    if (PQstatus(pg) == CONNECTION_BAD)
    {
        ast_log(LOG_ERROR, "Failed to connect to PostgreSQL database.\n");
        return -1;
    } else
        ast_log(LOG_DEBUG,"Successfully connected to PostgreSQL database.\n");

    res = ast_cdr_register(name, desc, pg_log);
    if (res)
        ast_log(LOG_ERROR, "Unable to register PostgreSQL CDR handling\n");

    return res;
}

int reload(void)
{
    unload_module();
    return load_module();
}

int usecount(void)
{
    return 0;
}

char *key()
{
    return ASTERISK_GPL_KEY;
}
